/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.midlet;

import java.io.*;
import java.util.*;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;

import com.funambol.syncclient.spds.SyncManagerFactory;

/**
 * This MIDlet implements a Program data store
 * with the following functionality:<br>
 * browsing data, update data (on device),
 *
  *
 * $Id: SyncClientDemoMIDlet.java,v 1.2 2006/01/27 13:48:42 fabius Exp $
 **/

public class SyncClientDemoMIDlet extends MIDlet
    implements CommandListener, ItemStateListener {
    //---------------------------------------------------------------- Constants

    final private static int    ERROR                     = 0               ;
    final private static int    INFO                      = 1               ;

    final private static int
        DELAY_TIME_FORM_NOTIFY_SYNC                       = 2000            ;

    final private static String KEY_SERVER_URL            = "server-url"    ;
    final private static String KEY_LOGIN                 = "login"         ;
    final private static String KEY_CHARSET               = "charset"       ;
    final private static String KEY_ENCODE                = "encode"        ;

    final private static String SOURCE_URI                = "briefcase"     ;

    final private static String DEFAULT_ENCODE            = "./false"       ;

    //---------------------------------------------------------------- Private data

    private Display display;

    private Command    cmdMain       = null;
    private Command    cmdSelect     = null;
    private Command    cmdExit       = null;

    private Command    cmdAdd        = null;
    private Command    cmdMod        = null;
    private Command    cmdDel        = null;
    private Command    cmdDbExit     = null;

    private Command    cmdBack       = null;
    private Command    cmdSave       = null;

    private String     serverUrl     = null;
    private String     login         = null;
    private String     charset       = null;
    private String     encode        = null;

    private Hashtable  properties    = null;

    private Form       urlForm       = null;

    private Form       syncForm      = null;

    private String[] mainScrChoices = {"Sync", "Display DB"};

    private javax.microedition.lcdui.List     mainScr     = null ;
    private javax.microedition.lcdui.List     dbScr       = null ;
    private javax.microedition.lcdui.TextBox  recordScr   = null ;

    private ChoiceGroup sortChoice  = null ;

    private int sortOrder = 1;

    //------------------------------------------------------------- Constructors

    /**
     * Called by the system to instantiate MIDlet.<br>
     * Caches reference to the display<br>
     * allocate commands.
     **/
    public SyncClientDemoMIDlet() {

        try {

            serverUrl  = getAppProperty(KEY_SERVER_URL );
            login      = getAppProperty(KEY_LOGIN      );
            charset    = getAppProperty(KEY_CHARSET    );
            encode     = getAppProperty(KEY_ENCODE     );

            properties = new Hashtable();

            properties.put(KEY_SERVER_URL, serverUrl  );
            properties.put(KEY_LOGIN,      login      );

            if (charset != null) {
                properties.put(KEY_CHARSET, charset);
            }

            if (encode != null && encode.length() > 0) {
                properties.put(KEY_ENCODE, encode);
            } else {
                properties.put(KEY_ENCODE, DEFAULT_ENCODE);
            }

        } catch (Exception e) {
            StaticDataHelper.log("Error: property not loaded: " + e.getMessage());
            midletExit();
        }

        display = Display.getDisplay(this);

        cmdMain    = new Command("Main",    Command.BACK, 2);
        cmdSelect  = new Command("Select",  Command.OK,   1);
        cmdExit    = new Command("Exit",    Command.EXIT, 1);

        cmdAdd     = new Command("Add",     Command.OK,   1);
        cmdMod     = new Command("Mod",     Command.OK,   1);
        cmdDel     = new Command("Del",     Command.OK,   1);
        cmdDbExit  = new Command("Exit",    Command.OK,   1);

        cmdBack    = new Command("Back",    Command.BACK, 2);
        cmdSave    = new Command("Save",    Command.OK,   1);

    }


    //------------------------------------------------------------- Protected methods

    /**
     * Called by the system to start MIDlet.
     * If the open of HttpDebugMIDlet fails, display
     * an alert and continue.
     **/
    protected void startApp() {
        genMainScr();
    }


    /**
     * Called by the system to pause MIDlet.
     * No actions required by our MIDlet.
     **/
    protected void pauseApp() {
    }


    /**
     * Called by the system to end our MIDlet.
     * No actions required by our MIDlet.
     **/
    protected void destroyApp(boolean unconditional) {
    }


    /**
     * Notify the system that we are exiting.
     */
    protected void midletExit() {
        destroyApp(true);
        notifyDestroyed();
    }


    //------------------------------------------------------------- Public methods

    /**
     * Create the main screen of MIDlet.
     * This screen is a list.
     */
    public Screen genMainScr() {

        String title = null;

        title = "Funambol J2ME demo";

        mainScr = new javax.microedition.lcdui.List(title,
                           javax.microedition.lcdui.List.IMPLICIT,
                           mainScrChoices,
                           null);

        mainScr.addCommand(cmdSelect);
        mainScr.addCommand(cmdExit);
        mainScr.setCommandListener(this);

        display.setCurrent(mainScr);

        return mainScr;
    }


    /**
     * Display DB
     * This screen is a list.
     **/
    public javax.microedition.lcdui.List displayDB() {

        String line = null;

        RecordStore recordStore  = null  ;
        String      record       = null  ;

        String      error        = null  ;
        boolean     loadDBError  = false ;

        Vector      records      = null  ;

        RecordEnumeration recordEnumeration = null;

        dbScr = new javax.microedition.lcdui.List("DB", javax.microedition.lcdui.List.IMPLICIT);

        try {

            recordStore = RecordStore.openRecordStore(SOURCE_URI, true);

            recordEnumeration = recordStore.enumerateRecords(null, null, false);

            while (recordEnumeration.hasNextElement()) {

                record = new String(recordEnumeration.nextRecord());

                dbScr.append(record, null);

            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            loadDBError = true;
            error = e.getMessage();
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            loadDBError = true;
            error = e.getMessage();
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
            loadDBError = true;
            error = e.getMessage();
        }

        if (loadDBError) {
            dbScr = new javax.microedition.lcdui.List("DB", javax.microedition.lcdui.List.IMPLICIT);
            dbScr.append("Datastore loading error: " + error, null);
        }

        if (dbScr.size() == 0) {
            dbScr.append("Datastore empty", null);
        }

        dbScr.addCommand(cmdMain);
        dbScr.addCommand(cmdAdd);
        dbScr.addCommand(cmdMod);
        dbScr.addCommand(cmdDel);
        dbScr.addCommand(cmdDbExit);
        dbScr.setCommandListener(this);
        display.setCurrent(dbScr);

        return dbScr;

    }


    /**
     * Display DB
     * This screen is a list.
     **/
    public javax.microedition.lcdui.TextBox displayRecord(int index) {

        String record    = null;
        String key       = null;
        String tmp       = null;
        String content   = null;

        String keyContent = null;

        if (index != -1) {
            record      = dbScr.getString(index)                     ;
            key         = record.substring(0, record.indexOf("|"))   ;
            tmp         = record.substring(record.indexOf("|") + 1)  ;
            content     = tmp.substring(tmp.indexOf("|") + 1)        ;
            keyContent  = key + "|" + content;
        } else {
            keyContent  = "||||";
        }

        recordScr = new javax.microedition.lcdui.TextBox("Record", keyContent, 650, TextField.ANY);

        recordScr.addCommand(cmdSave);
        recordScr.addCommand(cmdBack);

        recordScr.setCommandListener(this);
        display.setCurrent(recordScr);

        return recordScr;

    }


    /***
     * Implements a state machine that drives the
     * MIDlet from one state (screen) to the next.
     *
     * @param command command
     * @param disp source displayable
     **/

    public void commandAction(Command command,
                              Displayable disp) {

        int mainScrSelectedIndex;


        if (disp == mainScr) {

            if (command == cmdExit) {
                //
                // exitOption
                //
                midletExit();
            } else if ((command == javax.microedition.lcdui.List.SELECT_COMMAND) || (command == cmdSelect)) {

               mainScrSelectedIndex = mainScr.getSelectedIndex();

                switch (mainScrSelectedIndex) {
                    case 0:
                        //
                        // Sync
                        //
                        synchronize ();
                        break;
                    case 1:
                        //
                        // Display DB
                        //
                        displayDB();
                        break;
               }

            }

        }  else if (disp == dbScr) {

            if (command == cmdMain) {
                //
                // Display Main screen
                //
                genMainScr();
            } else if (command == cmdMod || (command == javax.microedition.lcdui.List.SELECT_COMMAND)) {
                //
                // Display Modify Record
                //
                int index = dbScr.getSelectedIndex();
                displayRecord(index);
            } else if (command == cmdAdd) {
                //
                // Display Add Record
                //
                displayRecord(-1);
            } else if (command == cmdDel) {
                //
                // Delete Record
                //
                int index = dbScr.getSelectedIndex();
                String record = dbScr.getString(index);
                deleteRecord(record);
                displayDB();
            } else if (command == cmdExit) {
                //
                // Display DB
                //
                displayDB();
            }

        } else if (disp == recordScr) {

            if (command == cmdBack) {
                //
                // Display DB
                //
                displayDB();
            } else if (command == cmdSave) {
                //
                // Save change
                //

                String record = recordScr.getString();
                storeRecord(record);
                displayDB();
            }

        } else if (disp == syncForm) {

            if (command == cmdMain) {
                //
                // Display Main screen
                //
                genMainScr();
            }

        }

    }


    /**
     * Gets called when the user is viewing the sort options<br>
     * in the exitOptionScr. Takes the new selected index and<br>
     * changes the sort order.
     *
     * @param item an item list
     **/
    public void itemStateChanged(Item item) {
        if (item == sortChoice) {
            sortOrder = sortChoice.getSelectedIndex();
        }
    }

    //------------------------------------------------------------- Private methods

    /**
     * Display wait synchronize form
     *
     **/
    private void synchronize() {

        Form waitForm = null;

        waitForm = new Form("Sync");

        waitForm.append(("\n\n  ...synchronizing..."));

        display.setCurrent(waitForm);

        Thread t = new Thread() {
            public void run() {
                sync();
            }
        };

        t.start();

        return;
    }


    /**
     * Synchronize data of <p>SOURCE_URI</p> recordStore
     * to SyncServer.
     * Connection setting are saved in JAD,
     * and stored in <p>properties</p>.
     *
     **/
    private void sync () {

        boolean syncError = false;

        try {
            SyncManagerFactory.getSyncManager(SOURCE_URI, properties).sync();
        } catch (Exception e) {
            syncError=true;
            StaticDataHelper.log("Error: " + e.getMessage());
        }

        syncForm = new Form("Synchronization");

        if (!syncError) {
            syncForm.append("\n" + "  Sync successful");
        } else {
            syncForm.append("\n" + "    Sync failed");
        }

        syncForm.addCommand(cmdMain);

        syncForm.setCommandListener(this);
        display.setCurrent(syncForm);

    }


    /**
     * Delete record to SOURCE_URI RecordStore
     *
     * @param record record to delet
     **/
    private void deleteRecord(String record) {

        String key     = record.substring(0, record.indexOf("|"))   ;
        String tmp     = record.substring(record.indexOf("|") + 1)  ;
        String content = tmp.substring(tmp.indexOf("|") + 1)        ;

        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        simpleFilter = new SimpleFilter(key + "|");

        int idRecord = -1;

        try {

            recordStore = RecordStore.openRecordStore(SOURCE_URI, true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {
                idRecord = recordEnumeration.nextRecordId();
            }

            if (idRecord != -1) {

                record = key     +
                         "|D|"   +
                         content ;

                recordStore.setRecord(idRecord, record.getBytes(), 0, (record.getBytes()).length);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
        }

    }


    /**
     * Add / update record to SOURCE_URI RecordStore
     *
     * @param keyContent string conteins key and content of record to add / update
     **/
    private void storeRecord(String keyContent) {

        String record = null;

        String key     = keyContent.substring(0, keyContent.indexOf("|"))  ;
        String content = keyContent.substring(keyContent.indexOf("|") + 1) ;

        RecordStore recordStore = null;

        SimpleFilter simpleFilter = null;

        RecordEnumeration recordEnumeration = null;

        simpleFilter = new SimpleFilter(key + "|");

        int idRecord = -1;

        try {

            recordStore = RecordStore.openRecordStore(SOURCE_URI, true);

            recordEnumeration = recordStore.enumerateRecords(simpleFilter, null, false);

            while (recordEnumeration.hasNextElement()) {
                idRecord = recordEnumeration.nextRecordId();
            }

            if (idRecord == -1) {

                record = key     +
                         "|N|"   +
                         content ;

                recordStore.addRecord(record.getBytes(), 0, (record.getBytes()).length);
            } else {

                record = key     +
                         "|U|"   +
                         content ;

                recordStore.setRecord(idRecord, record.getBytes(), 0, (record.getBytes()).length);
            }

            recordStore.closeRecordStore();

        } catch (javax.microedition.rms.InvalidRecordIDException e) {
            StaticDataHelper.log("error:" + e.getMessage());
        } catch (javax.microedition.rms.RecordStoreNotOpenException e) {
            StaticDataHelper.log("error:" + e.getMessage());
        } catch (javax.microedition.rms.RecordStoreException e) {
            StaticDataHelper.log("error:" + e.getMessage());
        }

    }


}